# -*- coding: utf-8 -*-
# Copyright (C) 2014-2024  EDF
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# See https://www.salome-platform.org/ or email : webmaster.salome@opencascade.com
#
# Author : Konstantin Leontev (OpenCascade S.A.S)

import sys

from qtsalome import QGridLayout, QFrame, QMessageBox, QApplication

from libGEOM_Swig import GEOM_Swig
from salome.geom import geomBuilder
from .geomrepairadv_logger import logger
from .basedlg import BaseDlg
from .geomrepairadv_common import DlgRef_1Spin_QTD
import GEOM

class MergeFacesDlg(BaseDlg):
    """
    Dialog for Merge Faces plugin that merges selected faces with a given precision.
    """

    def __init__(self, selection_level = GEOM.FACE):
        # Make layout for new widgets
        main_widget = QFrame()
        layout = QGridLayout(main_widget)
        layout.setContentsMargins(0, 0, 0, 0)

        # Precision widget
        self._precision_widget = DlgRef_1Spin_QTD()
        self._precision_widget.TextLabel1.setText('Precision')
        layout.addWidget(self._precision_widget, 0, 0)

        BaseDlg.__init__(
            self, main_widget, 'Merge Faces', 'merge_faces_algo.py', True, selection_level)


    def get_precision(self):
        """
        Returns current precision value.

        Args:
            None.

        Returns:
            Double.
        """

        return self._precision_widget.SpinBox_DX.value()


    def get_args(self):
        """
        Collects arguments for a repair execution algorithm into a dictionary.

        Args:
            None.

        Returns:
            Dictionary with arguments for execution.
        """

        geom_swig = GEOM_Swig()
        faces_ids = geom_swig.getLocalSelection()
        logger.debug('faces_ids: %s', faces_ids)

        if len(faces_ids) < 2:
            QMessageBox.warning(
                None,
                'Warning',
                'The algorithm needs at least two selected faces!\nMerging was canceled.'
                )
            return None

        # Get faces from a temporary copy object
        builder = geomBuilder.New()
        faces = builder.SubShapes(self._selected_copy, faces_ids)
        logger.debug('faces: %s', faces)

        return {
            'face_a': faces[0],
            'face_b': faces[1],
            'result_name': self.get_result_name(),
            'precision': self.get_precision()
        }


# For testing run as a module from geomrepairadv parent directory in
# Salome INSTALL, because the dialog needs a generated Ui_BaseDlg class
# that we don't have in the SOURCE.
# Example:
# $ python -m geomrepairadv.merge_faces
if __name__ == '__main__':
    app = QApplication(sys.argv)

    dlg = MergeFacesDlg(None)
    dlg.show()

    sys.exit(app.exec_())
